/*
 * F77 functions to be built into the root.exe executable.
 *
 * Copyright (c) 2000 Jacek M. Holeczek
 */

#include <stdio.h>
#include <stream.h>
#include "TROOT.h"
#include "Api.h"

#include "TF77.hxx"

ClassImp(TF77)

namespace F77 {
  
  extern "C" {
    
    /*  Define CERNLIB F77 common blocks for the different OSs, if needed. */
    
#if 0 /* define 1 if CERNLIB F77 commons are needed, 0 otherwise */
#define PAWC_SIZE 2000000
#ifndef WIN32
    int pawc_[PAWC_SIZE];
    int quest_[100];
    int hcbits_[37];
    int hcbook_[51];
    int rzcl_[11];
#else /* WIN32 */
#define pawc_   PAWC
#define quest_  QUEST
#define hcbits_ HCBITS
#define hcbook_ HCBOOK
#define rzcl_   RZCL
    extern int pawc_[PAWC_SIZE];
    extern int quest_[100];
    extern int hcbits_[37];
    extern int hcbook_[51];
    extern int rzcl_[11];
#endif /* WIN32 */
#endif /* CERNLIB F77 commons are needed */
    
    /* The interface to F77 functions and subroutines. */
    
#ifndef WIN32
#define type_of_call
#define DEFCHAR  const char*
#define PASSCHAR(string) string
#else /* WIN32 */
#define type_of_call  _stdcall
#define DEFCHAR  const char*, const int
#define PASSCHAR(string) string, strlen(string)
    
#define rrkstp_ RRKSTP
#define drkstp_ DRKSTP
#define rdeqbs_ RDEQBS
#define ddeqbs_ DDEQBS
#define rdeqmr_ RDEQMR
#define ddeqmr_ DDEQMR
    
#endif /* WIN32 */
    
    extern void type_of_call rrkstp_(Int_t &n, Float_t &h, Float_t &x, Float_t y[],
				     void (*sub)(Float_t &x, Float_t y[], Float_t f[]),
				     Float_t w[]);
    extern void type_of_call drkstp_(Int_t &n, Double_t &h, Double_t &x, Double_t y[],
				     void (*sub)(Double_t &x, Double_t y[], Double_t f[]),
				     Double_t w[]);
    extern void type_of_call rdeqbs_(Int_t &n, Float_t &x1, Float_t &x2, Float_t y[],
				     Float_t &h0, Float_t &eps,
				     void (*sub)(Float_t &x, Float_t y[], Float_t f[]),
				     Float_t w[]);
    extern void type_of_call ddeqbs_(Int_t &n, Double_t &x1, Double_t &x2, Double_t y[],
				     Double_t &h0, Double_t &eps,
				     void (*sub)(Double_t &x, Double_t y[], Double_t f[]),
				     Double_t w[]);
    extern void type_of_call rdeqmr_(Int_t &n, Float_t &x1, Float_t &x2, Float_t y[],
				     Float_t &h0, Float_t &eps,
				     void (*sub)(Float_t &x, Float_t y[], Float_t f[]),
				     Float_t w[]);
    extern void type_of_call ddeqmr_(Int_t &n, Double_t &x1, Double_t &x2, Double_t y[],
				     Double_t &h0, Double_t &eps,
				     void (*sub)(Double_t &x, Double_t y[], Double_t f[]),
				     Double_t w[]);
    
  } /* end of extern "C" */
  
} /* end of namespace F77 */

static void (*SUB_FX_FY_FF_POINTER)(Float_t, Float_t [], Float_t []) = 0;
static void SUB_FX_FY_FF(Float_t &X, Float_t Y[], Float_t F[])
{
  // # First define some local variables and objects. 
  G__CallFunc func; // INTERPRETEDFUNC, COMPILEDINTERFACEMETHOD, BYTECODEFUNC
  
  // # Then call the function
  switch(G__isinterpretedp2f(((void*)SUB_FX_FY_FF_POINTER))) {
    // # using function call as string 
  case G__INTERPRETEDFUNC:
    {
      char *fname;
      fname = G__p2f2funcname(((void*)SUB_FX_FY_FF_POINTER));
      if(fname) {
	G__ClassInfo globalscope;
	G__MethodInfo method;
	long dummy = 0;
	// resolve function overloading 
	method = globalscope.GetMethod(fname,
				       "Float_t, Float_t *, Float_t *",
				       &dummy);
	if(method.IsValid()) {
	  char temp[128];
	  sprintf(temp,"(Float_t)%e, (Float_t *)%p, (Float_t *)%p",
		  X, (void*)Y, (void*)F);
	  func.SetFunc(&globalscope,fname,temp,&dummy);
	  func.Exec((void*)NULL);
	  // get pointer to function again 
	  SUB_FX_FY_FF_POINTER = (void (*)(Float_t, Float_t *, Float_t *))method.PointerToFunc();
	} else cerr <<  "Error : Invalid method : " << *fname << endl;
      } else {
	cerr <<  "Error : Unknown pointer to function" << endl;
      }
    }
    break;
    // # using interface method 
  case G__COMPILEDINTERFACEMETHOD:
    func.SetFunc((G__InterfaceMethod)SUB_FX_FY_FF_POINTER);
    func.SetArg(X);
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # bytecode version of interpreted func 
  case G__BYTECODEFUNC:
    func.SetBytecode((struct G__bytecodefunc*)SUB_FX_FY_FF_POINTER);
    func.SetArg(X);
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # using true pointer to function 
  case G__COMPILEDTRUEFUNC:
    // # pointer not in CINT global function table 
  case G__UNKNOWNFUNC:
    (*SUB_FX_FY_FF_POINTER)(X, Y, F);
    break;
    // # this should never happen ( unknown kind of pointer ) 
  default:
    cerr << "Error : Unknown kind of pointer to function" << endl;
    break;
  }
}

static void (*SUB_DX_DY_DF_POINTER)(Double_t, Double_t [], Double_t []) = 0;
static void SUB_DX_DY_DF(Double_t &X, Double_t Y[], Double_t F[])
{
  // # First define some local variables and objects. 
  G__CallFunc func; // INTERPRETEDFUNC, COMPILEDINTERFACEMETHOD, BYTECODEFUNC
  
  // # Then call the function
  switch(G__isinterpretedp2f(((void*)SUB_DX_DY_DF_POINTER))) {
    // # using function call as string 
  case G__INTERPRETEDFUNC:
    {
      char *fname;
      fname = G__p2f2funcname(((void*)SUB_DX_DY_DF_POINTER));
      if(fname) {
	G__ClassInfo globalscope;
	G__MethodInfo method;
	long dummy = 0;
	// resolve function overloading 
	method = globalscope.GetMethod(fname,
				       "Double_t, Double_t *, Double_t *",
				       &dummy);
	if(method.IsValid()) {
	  char temp[128];
	  sprintf(temp,"(Double_t)%e, (Double_t *)%p, (Double_t *)%p",
		  X, (void*)Y, (void*)F);
	  func.SetFunc(&globalscope,fname,temp,&dummy);
	  func.Exec((void*)NULL);
	  // get pointer to function again 
	  SUB_DX_DY_DF_POINTER = (void (*)(Double_t, Double_t *, Double_t *))method.PointerToFunc();
	} else cerr <<  "Error : Invalid method : " << *fname << endl;
      } else {
	cerr <<  "Error : Unknown pointer to function" << endl;
      }
    }
    break;
    // # using interface method 
  case G__COMPILEDINTERFACEMETHOD:
    func.SetFunc((G__InterfaceMethod)SUB_DX_DY_DF_POINTER);
    func.SetArg(X);
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # bytecode version of interpreted func 
  case G__BYTECODEFUNC:
    func.SetBytecode((struct G__bytecodefunc*)SUB_DX_DY_DF_POINTER);
    func.SetArg(X);
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # using true pointer to function 
  case G__COMPILEDTRUEFUNC:
    // # pointer not in CINT global function table 
  case G__UNKNOWNFUNC:
    (*SUB_DX_DY_DF_POINTER)(X, Y, F);
    break;
    // # this should never happen ( unknown kind of pointer ) 
  default:
    cerr << "Error : Unknown kind of pointer to function" << endl;
    break;
  }
}

// First-order Differential Equations (Runge-Kutta) - MATHLIB D200 

void TF77::rkstp(Int_t &N, Float_t &H, Float_t &X, Float_t Y[],
		 void (*SUB)(Float_t X, Float_t Y[], Float_t F[]),
		 Float_t W[])
{
  void (*SUB_FX_FY_FF_OLD_POINTER)(Float_t, Float_t [], Float_t []);
  SUB_FX_FY_FF_OLD_POINTER = SUB_FX_FY_FF_POINTER;
  SUB_FX_FY_FF_POINTER = SUB;
  F77::rrkstp_(N, H, X, Y, SUB_FX_FY_FF, W);
  SUB_FX_FY_FF_POINTER = SUB_FX_FY_FF_OLD_POINTER;
}

void TF77::rkstp(Int_t &N, Double_t &H, Double_t &X, Double_t Y[],
		 void (*SUB)(Double_t X, Double_t Y[], Double_t F[]),
		 Double_t W[])
{
  void (*SUB_DX_DY_DF_OLD_POINTER)(Double_t, Double_t [], Double_t []);
  SUB_DX_DY_DF_OLD_POINTER = SUB_DX_DY_DF_POINTER;
  SUB_DX_DY_DF_POINTER = SUB;
  F77::drkstp_(N, H, X, Y, SUB_DX_DY_DF, W);
  SUB_DX_DY_DF_POINTER = SUB_DX_DY_DF_OLD_POINTER;
}

// First-order Differential Equations (Gragg-Bulirsch-Stoer) - MATHLIB D201 

void TF77::deqbs(Int_t &N, Float_t &X1, Float_t &X2, Float_t Y[],
		 Float_t &H0, Float_t &EPS,
		 void (*SUB)(Float_t X, Float_t Y[], Float_t F[]),
		 Float_t W[])
{
  void (*SUB_FX_FY_FF_OLD_POINTER)(Float_t, Float_t [], Float_t []);
  SUB_FX_FY_FF_OLD_POINTER = SUB_FX_FY_FF_POINTER;
  SUB_FX_FY_FF_POINTER = SUB;
  F77::rdeqbs_(N, X1, X2, Y, H0, EPS, SUB_FX_FY_FF, W);
  SUB_FX_FY_FF_POINTER = SUB_FX_FY_FF_OLD_POINTER;
}

void TF77::deqbs(Int_t &N, Double_t &X1, Double_t &X2, Double_t Y[],
		 Double_t &H0, Double_t &EPS,
		 void (*SUB)(Double_t X, Double_t Y[], Double_t F[]),
		 Double_t W[])
{
  void (*SUB_DX_DY_DF_OLD_POINTER)(Double_t, Double_t [], Double_t []);
  SUB_DX_DY_DF_OLD_POINTER = SUB_DX_DY_DF_POINTER;
  SUB_DX_DY_DF_POINTER = SUB;
  F77::ddeqbs_(N, X1, X2, Y, H0, EPS, SUB_DX_DY_DF, W);
  SUB_DX_DY_DF_POINTER = SUB_DX_DY_DF_OLD_POINTER;
}

// First-order Differential Equations (Runge-Kutta-Merson) - MATHLIB D202 

void TF77::deqmr(Int_t &N, Float_t &X1, Float_t &X2, Float_t Y[],
		 Float_t &H0, Float_t &EPS,
		 void (*SUB)(Float_t X, Float_t Y[], Float_t F[]),
		 Float_t W[])
{
  void (*SUB_FX_FY_FF_OLD_POINTER)(Float_t, Float_t [], Float_t []);
  SUB_FX_FY_FF_OLD_POINTER = SUB_FX_FY_FF_POINTER;
  SUB_FX_FY_FF_POINTER = SUB;
  F77::rdeqmr_(N, X1, X2, Y, H0, EPS, SUB_FX_FY_FF, W);
  SUB_FX_FY_FF_POINTER = SUB_FX_FY_FF_OLD_POINTER;
}

void TF77::deqmr(Int_t &N, Double_t &X1, Double_t &X2, Double_t Y[],
		 Double_t &H0, Double_t &EPS,
		 void (*SUB)(Double_t X, Double_t Y[], Double_t F[]),
		 Double_t W[])
{
  void (*SUB_DX_DY_DF_OLD_POINTER)(Double_t, Double_t [], Double_t []);
  SUB_DX_DY_DF_OLD_POINTER = SUB_DX_DY_DF_POINTER;
  SUB_DX_DY_DF_POINTER = SUB;
  F77::ddeqmr_(N, X1, X2, Y, H0, EPS, SUB_DX_DY_DF, W);
  SUB_DX_DY_DF_POINTER = SUB_DX_DY_DF_OLD_POINTER;
}

/* end of file TF77.cxx */
