#ifndef TF77_HXX
#define TF77_HXX

#include "Rtypes.h"

class TF77 {
public:
  
  // First-order Differential Equations (Runge-Kutta) - MATHLIB D200 
  
  static void rkstp(Int_t &N, Float_t &H, Float_t &X, Float_t Y[],
		    void (*SUB)(Float_t X, Float_t Y[], Float_t F[]),
		    Float_t W[]);
  
  static void rkstp(Int_t &N, Double_t &H, Double_t &X, Double_t Y[],
		    void (*SUB)(Double_t X, Double_t Y[], Double_t F[]),
		    Double_t W[]);
  
  // First-order Differential Equations (Gragg-Bulirsch-Stoer) - MATHLIB D201 
  
  static void deqbs(Int_t &N, Float_t &X1, Float_t &X2, Float_t Y[],
		    Float_t &H0, Float_t &EPS,
		    void (*SUB)(Float_t X, Float_t Y[], Float_t F[]),
		    Float_t W[]);
  
  static void deqbs(Int_t &N, Double_t &X1, Double_t &X2, Double_t Y[],
		    Double_t &H0, Double_t &EPS,
		    void (*SUB)(Double_t X, Double_t Y[], Double_t F[]),
		    Double_t W[]);
  
  // First-order Differential Equations (Runge-Kutta-Merson) - MATHLIB D202 
  
  static void deqmr(Int_t &N, Float_t &X1, Float_t &X2, Float_t Y[],
		    Float_t &H0, Float_t &EPS,
		    void (*SUB)(Float_t X, Float_t Y[], Float_t F[]),
		    Float_t W[]);
  
  static void deqmr(Int_t &N, Double_t &X1, Double_t &X2, Double_t Y[],
		    Double_t &H0, Double_t &EPS,
		    void (*SUB)(Double_t X, Double_t Y[], Double_t F[]),
		    Double_t W[]);
  
public:
  TF77(); // Default ctor used to initialize F77 resources
  virtual ~TF77(); // Default dtor used to clean up and free F77 resources
  
  ClassDef(TF77,0)  // C++ glue code for F77 functions and subroutines
};

R__EXTERN TF77 *gF77; // Single, global instance of the TF77 object

#endif /* TF77_HXX */

/* end of file TF77.hxx */
