#include <stdio.h>
#include <math.h>

void sub(double x, double y[], double f[])
{
  
#if 0
  printf("&x=%p, x=%e, y=%p, f=%p\n", &x, x, y, f);
  fflush(stdout);
#endif
  
  f[0] = y[0] - 3.0 * y[1];
  f[1] = 3.0 * y[0] + y[1];
}

#if 0

// ROOT macro testing RKSTP, DEQBS, DEQMR (MATHLIB D200, D201, D202)

gROOT->Reset();

// Load the "sub" function as an interpreted, or compiled function
// .L dtest.cxx
.L dtest.cxx++

double x1 = 0.0;
double x2 = 3.0;
double h = (x2 - x1) / 100.0;
int n = 2;
double x = x1;
double y[n] = {1.0, -1.0};
double w[36*n];
int in = ((int)((x2 - x1) / h + 0.5)) + 1;
double x01[in];
double y0[in], y1[in];
double h0 = h;
double eps = 0.001;
int i;

// Prepare Canvas and Pads
TCanvas *c0 = new TCanvas("c0","Numerical Solving Example");
TPad *pad0 = new TPad("pad0","y[0] (RKSTP)", 0.01, 0.67, 0.50, 0.99);
TPad *pad1 = new TPad("pad1","y[1] (RKSTP)", 0.51, 0.67, 0.99, 0.99);
TPad *pad2 = new TPad("pad2","y[0] (DEQBS)", 0.01, 0.34, 0.50, 0.66);
TPad *pad3 = new TPad("pad3","y[1] (DEQBS)", 0.51, 0.34, 0.99, 0.66);
TPad *pad4 = new TPad("pad4","y[0] (DEQMR)", 0.01, 0.01, 0.50, 0.33);
TPad *pad5 = new TPad("pad5","y[1] (DEQMR)", 0.51, 0.01, 0.99, 0.33);
pad0->Draw();
pad1->Draw();
pad2->Draw();
pad3->Draw();
pad4->Draw();
pad5->Draw();

// Exact solutions (for initial conditions x=0, y[0]=1, y[1]=-1)
TF1 *ey0 = new TF1("ey0", "exp(x) * (sin(3*x) + cos(3*x))", x1, x2);
TF1 *ey1 = new TF1("ey1", "exp(x) * (sin(3*x) - cos(3*x))", x1, x2);



// Test ***** RKSTP - MATHLIB D200 *****



// Set initial values
x = 0.0;
y[0] = 1.0;
y[1] = -1.0;

// Due to problems in the current CINT (as of cint version 5.14.50,
// embedded in root version 2.25/03), if the "sub" function is an
// interpreted function one needs to try to ByteCompile it (this
// problem was not present in older versions of CINT, let's hope
// it will disappear in newer versions, as well - this is not needed
// if "sub" is a precompiled function).
// To achieve it, it is sufficient to call "sub" once in a "for" loop.
// In case ByteCompilation fails, you may later get interpreter errors
// "Limitation: Precompiled function can not get pointer to interpreted
// function as argument". Don't worry, just in all calls to RKSTP, DEQBS
// and DEQMR below, replace the "sub" parameter with "(void *)sub".
if (G__isinterpretedp2f((void*)sub) == 1) for (i = 0; i < 1; i++) sub(x, y, w);

// Solve equations (creates also x01[] table used in DEQBS, DEQMR tests below)
x01[0] = x;
y0[0] = y[0];
y1[0] = y[1];
for (i = 1; i < in; i++) {TF77::rkstp(n, h, x, y, sub, w); x01[i] = x; y0[i] = y[0]; y1[i] = y[1];}

// Draw solutions (exact and numerical)
pad0->cd();
TGraph *g0 = new TGraph(in, x01, y0);
g0->SetTitle(pad0->GetTitle());
g0->Draw("A*");
ey0->Draw("SAME");
pad1->cd();
TGraph *g1 = new TGraph(in, x01, y1);
g1->SetTitle(pad1->GetTitle());
g1->Draw("A*");
ey1->Draw("SAME");



// Test ***** DEQBS - MATHLIB D201 *****



// First just a small test -  calculate values in point x2
y[0] = 1.0;
y[1] = -1.0;
h0 = (x2-x1)/10.0;
eps = 0.000001;
TF77::deqbs(n, x1, x2, y, h0, eps, sub, w);
printf("h0 = %e, x2 = %e\n", h0, x2);
printf("y[0] = %e, ey0 = %e\n", y[0], ey0->Eval(x2, 0.0, 0.0));
printf("y[1] = %e, ey1 = %e\n", y[1], ey1->Eval(x2, 0.0, 0.0));

// Set initial values
y[0] = 1.0;
y[1] = -1.0;

// Solve equations (uses x01[] table created in RKSTP test above)
h0 = h / 2.0;
eps = 0.000001;
y0[0] = y[0];
y1[0] = y[1];
for (i = 1; i < in; i++) {TF77::deqbs(n, x01[i-1], x01[i], y, h0, eps, sub, w); y0[i] = y[0]; y1[i] = y[1];}

// Draw solutions (exact and numerical)
pad2->cd();
TGraph *g2 = new TGraph(in, x01, y0);
g2->SetTitle(pad2->GetTitle());
g2->Draw("A*");
ey0->Draw("SAME");
pad3->cd();
TGraph *g3 = new TGraph(in, x01, y1);
g3->SetTitle(pad3->GetTitle());
g3->Draw("A*");
ey1->Draw("SAME");



// Test ***** DEQMR - MATHLIB D202 *****



// First just a small test -  calculate values in point x2
y[0] = 1.0;
y[1] = -1.0;
h0 = (x2-x1)/10.0;
eps = 0.000001;
TF77::deqmr(n, x1, x2, y, h0, eps, sub, w);
printf("h0 = %e, x2 = %e\n", h0, x2);
printf("y[0] = %e, ey0 = %e\n", y[0], ey0->Eval(x2, 0.0, 0.0));
printf("y[1] = %e, ey1 = %e\n", y[1], ey1->Eval(x2, 0.0, 0.0));

// Set initial values
y[0] = 1.0;
y[1] = -1.0;

// Solve equations (uses x01[] table created in RKSTP test above)
h0 = h / 2.0;
eps = 0.000001;
y0[0] = y[0];
y1[0] = y[1];
for (i = 1; i < in; i++) {TF77::deqmr(n, x01[i-1], x01[i], y, h0, eps, sub, w); y0[i] = y[0]; y1[i] = y[1];}

// Draw solutions (exact and numerical)
pad4->cd();
TGraph *g4 = new TGraph(in, x01, y0);
g4->SetTitle(pad4->GetTitle());
g4->Draw("A*");
ey0->Draw("SAME");
pad5->cd();
TGraph *g5 = new TGraph(in, x01, y1);
g5->SetTitle(pad5->GetTitle());
g5->Draw("A*");
ey1->Draw("SAME");

c0->Update();

#endif
