#ifndef ROOT_TInverse
#define ROOT_TInverse

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TInverse                                                             //
//                                                                      //
// Encapsulate matrix inversion routines.                               //
// C++ Implementation of original code by Sigmund Brandt in             //
// "Data analysis: Statistical and Computational Methods for Scientists //
//  and Engineers"                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include <TROOT.h>
#include <Rtypes.h>
#include <TError.h>
#include <TArrayD.h>

class TInverse
{

private:
  static TArrayD GetColumn       (TArrayD &a,const Int_t m,const Int_t n,const Int_t i);
  static void    PutColumn       (TArrayD &a,TArrayD &c,const Int_t m,const Int_t n,const Int_t i);
  static TArrayD GetRow          (TArrayD &a,const Int_t n,const Int_t i);
  static void    PutRow          (TArrayD &a,TArrayD &r,const Int_t n,const Int_t i);
  static void    DefGivens       (const Double_t v1,const Double_t v2,Double_t &c,Double_t &s);
  static void    DefAplGivens    (Double_t &v1,Double_t &v2,Double_t &c,Double_t &s);
  static void    ApplyGivens     (Double_t &z1,Double_t &z2,const Double_t c,const Double_t s);
  static void    DefHouseHolder  (TArrayD &v,Double_t &up,Double_t &b,const Int_t lp,const Int_t l);
  static void    ApplyHouseHolder(TArrayD &v,const Double_t up,const Double_t b,TArrayD &c,
                                  const Int_t lp,const Int_t l);
  static TArrayD GetSubMat       (TArrayD &a,const Int_t n,const Int_t k,const Int_t l,
                                  const Int_t m0,const Int_t n0);
  static void    PutSubMat       (TArrayD &a,TArrayD &s,const Int_t n,const Int_t k,const Int_t l,
                                  const Int_t m0,const Int_t n0,const Double_t fact);

  static void    Sv1             (TArrayD &a,TArrayD &b,TArrayD &d,TArrayD &e,const Int_t m,
                                  const Int_t n,const Int_t nb);
  static Int_t   Sv2             (TArrayD &a,TArrayD &b,TArrayD &d,TArrayD &e,const Int_t m,
                                  const Int_t n,const Int_t nb);
  static void    Sv2_1           (TArrayD &a,TArrayD &d,TArrayD &e,const Int_t n,const Int_t k);
  static void    Sv2_2           (TArrayD &b,TArrayD &d,TArrayD &e,const Int_t nb,const Int_t k,
                                  const Int_t l);
  static void    Sv2_3           (TArrayD &a,TArrayD &b,TArrayD &d,TArrayD &e,const Int_t n,
                                  const Int_t nb,const Int_t k,const Int_t l);
  static void    Sv3             (TArrayD &a,TArrayD &b,TArrayD &d,Int_t m,const Int_t n,
                                  const Int_t nb);
  static Int_t   Sv4             (TArrayD &a,TArrayD &b,TArrayD &d,TArrayD &x,TArrayD &r,
                                  const Int_t m,const Int_t n,const Int_t nb,const Double_t frac);

  static Int_t   LUDecompose     (Double_t *a,Int_t n,Int_t *index,Double_t d);
  static void    LUBacksub       (Double_t *a,Int_t n,Int_t *index);
  static Int_t   Pdcholesky      (Double_t *a,Double_t *u,Int_t n);

public:

  static Int_t   PseudoInv       (TArrayD &a,const Int_t m,const Int_t n,const Double_t frac,
                                  Int_t &nrSingular);
  static Int_t   SvDeComp        (TArrayD &a,TArrayD &b,TArrayD &x,TArrayD &r,const Int_t m,
                                  const Int_t n,const Double_t frac,Int_t &nrSingular);
  static Int_t   SvDeCompBound   (TArrayD &a,TArrayD &b,TArrayD &e,TArrayD &d,TArrayD &x,
                                  TArrayD &r,const Int_t m,const Int_t n,const Int_t l,
                                  const Double_t frac,Int_t &nrSingular);
  static Int_t   Inverse         (TArrayD &a);
  static Int_t   InversePosDef   (TArrayD &a);

  ClassDef(TInverse,0)  //Matrix inversion Utilities
};

#endif
