/*
 * CERNLIB functions to be built into the root.exe executable.
 *
 * Copyright (c) 2000 Jacek M. Holeczek
 */

#include <stdio.h>
#include <stream.h>
#include "TROOT.h"
#include "Api.h"

#include "CERNfunctions.hxx"

extern "C" {
  
  /* The interface to CERNLIB FORTRAN functions. */
  
#ifdef extname
#define rrkstp rrkstp_
#define drkstp drkstp_
#define rdeqbs rdeqbs_
#define ddeqbs ddeqbs_
#define rdeqmr rdeqmr_
#define ddeqmr ddeqmr_
#endif
  
  extern void rrkstp(Int_t &n, Float_t &h, Float_t &x, Float_t *y,
		     void (*sub)(Float_t &x, Float_t *y, Float_t *f),
		     Float_t *w);
  extern void drkstp(Int_t &n, Double_t &h, Double_t &x, Double_t *y,
		     void (*sub)(Double_t &x, Double_t *y, Double_t *f),
		     Double_t *w);
  extern void rdeqbs();
  extern void ddeqbs();
  extern void rdeqmr();
  extern void ddeqmr();
  
} /* end of extern "C" */

static void (*SUB_FX_FY_FF_POINTER)(Float_t &X, Float_t *Y, Float_t *F) = 0;
static void SUB_FX_FY_FF(Float_t &X, Float_t *Y, Float_t *F)
{
  // # First define some local variables and objects. 
  char temp[128]; // INTERPRETEDFUNC
  long offset = 0; // INTERPRETEDFUNC
  G__ClassInfo globalscope; // INTERPRETEDFUNC
  G__CallFunc func; // INTERPRETEDFUNC, COMPILEDINTERFACEMETHOD, BYTECODEFUNC
  
  // # Then call the function
  switch(G__isinterpretedp2f(((void*)SUB_FX_FY_FF_POINTER))) {
    // # using function call as string 
  case G__INTERPRETEDFUNC:
    sprintf(temp,"(Float_t &)%p, (Float_t *)%p, (Float_t *)%p",
	    (void*)&X, (void*)Y, (void*)F);
    func.SetFunc(&globalscope,(char*)SUB_FX_FY_FF_POINTER,temp,&offset);
    func.Exec((void*)NULL);
    break;
    // # using interface method 
  case G__COMPILEDINTERFACEMETHOD:
    func.SetFunc((G__InterfaceMethod)SUB_FX_FY_FF_POINTER);
    func.SetArg(((long)&X));
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # bytecode version of interpreted func 
  case G__BYTECODEFUNC:
    func.SetBytecode((struct G__bytecodefunc*)SUB_FX_FY_FF_POINTER);
    func.SetArg(((long)&X));
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # using true pointer to function 
  case G__COMPILEDTRUEFUNC:
    // # pointer not in CINT global function table 
  case G__UNKNOWNFUNC:
    (*SUB_FX_FY_FF_POINTER)(X, Y, F);
    break;
    // # this should never happen ( unknown kind of pointer ) 
  default:
    cerr << "Error : Unknown kind of pointer to function" << endl;
    break;
  }
}

void RKSTP(Int_t &N, Float_t &H, Float_t &X, Float_t *Y,
	   void (*SUB)(Float_t &X, Float_t *Y, Float_t *F),
	   Float_t *W)
{
  void (*SUB_FX_FY_FF_OLD_POINTER)(Float_t &X, Float_t *Y, Float_t *F);
  SUB_FX_FY_FF_OLD_POINTER = SUB_FX_FY_FF_POINTER;
  SUB_FX_FY_FF_POINTER = SUB;
  rrkstp(N, H, X, Y, SUB_FX_FY_FF, W);
  SUB_FX_FY_FF_POINTER = SUB_FX_FY_FF_OLD_POINTER;
}

static void (*SUB_DX_DY_DF_POINTER)(Double_t &X, Double_t *Y, Double_t *F) = 0;
static void SUB_DX_DY_DF(Double_t &X, Double_t *Y, Double_t *F)
{
  // # First define some local variables and objects. 
  char temp[128]; // INTERPRETEDFUNC
  long offset = 0; // INTERPRETEDFUNC
  G__ClassInfo globalscope; // INTERPRETEDFUNC
  G__CallFunc func; // INTERPRETEDFUNC, COMPILEDINTERFACEMETHOD, BYTECODEFUNC
  
  // # Then call the function
  switch(G__isinterpretedp2f(((void*)SUB_DX_DY_DF_POINTER))) {
    // # using function call as string 
  case G__INTERPRETEDFUNC:
    sprintf(temp,"(Double_t &)%p, (Double_t *)%p, (Double_t *)%p",
	    (void*)&X, (void*)Y, (void*)F);
    func.SetFunc(&globalscope,(char*)SUB_DX_DY_DF_POINTER,temp,&offset);
    func.Exec((void*)NULL);
    break;
    // # using interface method 
  case G__COMPILEDINTERFACEMETHOD:
    func.SetFunc((G__InterfaceMethod)SUB_DX_DY_DF_POINTER);
    func.SetArg(((long)&X));
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # bytecode version of interpreted func 
  case G__BYTECODEFUNC:
    func.SetBytecode((struct G__bytecodefunc*)SUB_DX_DY_DF_POINTER);
    func.SetArg(((long)&X));
    func.SetArg(((long)Y));
    func.SetArg(((long)F));
    func.Exec((void*)NULL);
    break;
    // # using true pointer to function 
  case G__COMPILEDTRUEFUNC:
    // # pointer not in CINT global function table 
  case G__UNKNOWNFUNC:
    (*SUB_DX_DY_DF_POINTER)(X, Y, F);
    break;
    // # this should never happen ( unknown kind of pointer ) 
  default:
    cerr << "Error : Unknown kind of pointer to function" << endl;
    break;
  }
}

void RKSTP(Int_t &N, Double_t &H, Double_t &X, Double_t *Y,
	   void (*SUB)(Double_t &X, Double_t *Y, Double_t *F),
	   Double_t *W)
{
  void (*SUB_DX_DY_DF_OLD_POINTER)(Double_t &X, Double_t *Y, Double_t *F);
  SUB_DX_DY_DF_OLD_POINTER = SUB_DX_DY_DF_POINTER;
  SUB_DX_DY_DF_POINTER = SUB;
  drkstp(N, H, X, Y, SUB_DX_DY_DF, W);
  SUB_DX_DY_DF_POINTER = SUB_DX_DY_DF_OLD_POINTER;
}

/* end of file CERNfunctions.cxx */
